﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Media;
using System.Linq;
using ff14bot;
using ff14bot.Helpers;
using ff14bot.Interfaces;
using ff14bot.Managers;
using ff14bot.NeoProfiles;
using ff14bot.Navigation;
using ff14bot.RemoteWindows;
using Newtonsoft.Json;
using Clio.Utilities;
using CrystalHunter.Settings;

namespace CrystalHunter
{
    public class CrystalHunter : IBotPlugin
    {
        public string 	Author 		{ get { return "Zee"; } }
        public Version 	Version 	{ get { return new Version(1, 3); } }
        public string 	Name 		{ get { return "CrystalHunter"; } }
        public string 	Description	{ get { return "Grind FATES for Anima Relic Stage 1 Crystals. Concept based on Atma Hunter by ExMortem: https://www.thebuddyforum.com/rebornbuddy-forum/plugins/169111-atma-hunter.html"; } }

		public static readonly Color LogColor = Color.FromRgb(0,255,255);
		
        public static CrystalHunterSettings settings = CrystalHunterSettings.instance; 
		
        private Form _configForm;
        
		public bool WantButton
        {
            get { return true; }
        }
		
        public string ButtonText
        {
            get { return "CrystalHunter"; }
        }

		public void OnButtonPress()
        {
            if (_configForm == null || _configForm.IsDisposed || _configForm.Disposing)
                _configForm = new CrystalHunterSettingsForm();

            _configForm.ShowDialog();
        }

        public bool Equals(IBotPlugin other)
        {
            throw new NotImplementedException();
        }

        public void OnPulse()
        {
            if ( (BotManager.Current.Name == "Fate Bot" || BotManager.Current.Name == "ExFateBot") && !Finished)
				Grind();
        }

		private bool Finished 					= false;
		
		#region Crystal Consts
		private const string IceCrystal 		= "Luminous Ice Crystal";
		private const string WaterCrystal 		= "Luminous Water Crystal";
		private const string WindCrystal 		= "Luminous Wind Crystal";
		private const string LightningCrystal 	= "Luminous Lightning Crystal";
		private const string EarthCrystal 		= "Luminous Earth Crystal";
		private const string FireCrystal 		= "Luminous Fire Crystal";
		
		private const string UmbralNodule		= "Umbral Nodule";
		private const string AstralNodule		= "Astral Nodule";
		#endregion
		
		#region Locations Consts
		private const string FalconsNest 		= "falcon's nest";
		private const string CampCloudtop 		= "camp cloudtop";
		private const string Helix 				= "helix";
		private const string Idyllshire 		= "idyllshire";
		private const string TailFeather 		= "tailfeather";
		private const string Moghome 			= "moghome";
		#endregion

		private int CrystalCount;

        Dictionary<int, string> CrystalDictionary 				= new Dictionary<int, string>();
        Dictionary<int, string> ZoneDictionary 					= new Dictionary<int, string>();
        Dictionary<int, string> LocationDictionary 				= new Dictionary<int, string>();
        Dictionary<string, string> CrystalLocationDictionary	= new Dictionary<string, string>();

        Dictionary<int, string> UmbralNoduleDictionary 			= new Dictionary<int, string>();
        Dictionary<int, string> AstralNoduleDictionary 			= new Dictionary<int, string>();

        public void OnInitialize()
        {
			Finished = false;

            #region Nodule Dictionary
			UmbralNoduleDictionary.Add(0, IceCrystal);
			UmbralNoduleDictionary.Add(1, EarthCrystal);
			UmbralNoduleDictionary.Add(2, WaterCrystal);
			
			AstralNoduleDictionary.Add(0, FireCrystal);
			AstralNoduleDictionary.Add(1, WindCrystal);
			AstralNoduleDictionary.Add(2, LightningCrystal);
            #endregion
			
			#region Crystal Dictionary
			CrystalDictionary.Add(50, IceCrystal);
			CrystalDictionary.Add(52, EarthCrystal);
			CrystalDictionary.Add(54, WindCrystal);
			CrystalDictionary.Add(56, LightningCrystal);
			CrystalDictionary.Add(57, WaterCrystal);
			CrystalDictionary.Add(59, FireCrystal);
            #endregion
			
			#region Location Dictionary
			LocationDictionary.Add(71, FalconsNest);
			LocationDictionary.Add(72, CampCloudtop);
			LocationDictionary.Add(74, Helix);
			LocationDictionary.Add(75, Idyllshire);
			LocationDictionary.Add(76, TailFeather);
			LocationDictionary.Add(78, Moghome);
            #endregion
			
			#region Zone Dictionary
			ZoneDictionary.Add(397, FalconsNest);	//"Coerthas Western Highlands"
			ZoneDictionary.Add(398, TailFeather);	//"Dravanian Forelands"
			ZoneDictionary.Add(399, Idyllshire);	//"Dravanian Hinterlands"
			ZoneDictionary.Add(400, Moghome);		//"Churning Mists"
			ZoneDictionary.Add(401, CampCloudtop);	//"Sea of Clouds"
			ZoneDictionary.Add(402, Helix);			//"Azys Lla"
            #endregion

			#region Crystal Location Dictionary
			CrystalLocationDictionary.Add(IceCrystal, FalconsNest);
			CrystalLocationDictionary.Add(EarthCrystal, TailFeather);
			CrystalLocationDictionary.Add(WindCrystal, CampCloudtop);
			CrystalLocationDictionary.Add(LightningCrystal, Moghome);
			CrystalLocationDictionary.Add(WaterCrystal, Idyllshire);
			CrystalLocationDictionary.Add(FireCrystal, Helix);
			#endregion

			CrystalCount = settings.CrystalCount;
        }
        public void OnShutdown(){ }
		
        public void OnEnabled()
        {
			Logging.Write(LogColor, "[Crystal Hunter] Enabled.");
			
			Logging.Write(LogColor, "[Crystal Hunter] Aiming for {0} of each crystal", CrystalCount);
			
			InventoryScan();
        }

        public void OnDisabled()
        {
			Logging.Write(LogColor, "[Crystal Hunter] Disabled.");
        }

        #region Crystals
        public void Grind()
        {
			if (String.IsNullOrEmpty(GrindLocation()))
			{
				Logging.Write(LogColor, "[Crystal Hunter] Looks like you've finished grinding, congratulations!");
				Finished = true;
				return;
			}

			if (InIdyllshire())
			{
				MoveIntoDravanianHinterlands();
				return;
			}

			if (AreWeInGrindZone())
				return;

			TreeRoot.StopGently("[Crystal Hunter] Moving to " + GrindLocation());

			Logging.Write(LogColor, "[Crystal Hunter] Moving to " + GrindLocation());
			Logging.Write(LogColor, "[Crystal Hunter] Going after " + GetCrystalForLocation(GrindLocation()));

			WorldManager.TeleportById((uint)GrindLocationId());

			Logging.Write(LogColor, "[Crystal Hunter] Will now wait for 15 seconds for transport to complete.");
			Thread.Sleep(15000);

			TreeRoot.Start();
        }
		
		public List<string> InventoryScan()
		{
			List<String> inventory = new List<String>();

			foreach(string crystal in CrystalDictionary.Values)
			{
				inventory.Add(String.Format("{0} {1} in inventory.", ConditionParser.ItemCount(crystal), crystal));
				Logging.Write(LogColor, "[Crystal Hunter] {0} {1} in inventory.", ConditionParser.ItemCount(crystal), crystal);
			}

			inventory.Add(String.Format("{0} {1} found within inventory.", ConditionParser.ItemCount(UmbralNodule), UmbralNodule));
			Logging.Write(LogColor, "[Crystal Hunter] {0} {1} found within inventory.", ConditionParser.ItemCount(UmbralNodule), UmbralNodule);

			inventory.Add(String.Format("{0} {1} found within inventory.", ConditionParser.ItemCount(AstralNodule), AstralNodule));
			Logging.Write(LogColor, "[Crystal Hunter] {0} {1} found within inventory.", ConditionParser.ItemCount(AstralNodule), AstralNodule);

			return inventory;
		}
		
		private bool DoesInventoryContainUmbralNodule()
		{
			int itemCount = ConditionParser.ItemCount(UmbralNodule);
			
			if (settings.VerboseLogging)
				Logging.Write(LogColor, "[Crystal Hunter] {0} {1} in inventory.", itemCount, UmbralNodule);

            return (itemCount > 0) ? true : false;
		}
		
		private bool DoesInventoryContainAstralNodule()
		{
			int itemCount = ConditionParser.ItemCount(AstralNodule);
			
			if (settings.VerboseLogging)
				Logging.Write(LogColor, "[Crystal Hunter] {0} {1} in inventory.", itemCount, AstralNodule);

            return (itemCount > 0) ? true : false;
		}

		private bool DoesInventoryContainCrystal(string crystal)
		{
			int itemCount = ConditionParser.ItemCount(crystal);
			
			if (settings.VerboseLogging)
				Logging.Write(LogColor, "[Crystal Hunter] {0} {1} in inventory.", itemCount, crystal);

            return (itemCount >= CrystalCount) ? true : false;
		}

		private string GetCrystalToGather()
		{
			foreach (string crystal in CrystalDictionary.Values)
            {
				if (settings.NoduleCheck)
				{
					if (DoesInventoryContainUmbralNodule() && UmbralNoduleDictionary.ContainsValue(crystal))
						break;
					
					if (DoesInventoryContainAstralNodule() && AstralNoduleDictionary.ContainsValue(crystal))
						break;
				}
				
                if (!DoesInventoryContainCrystal(crystal))
                    return crystal;
            }
			
			return String.Empty;
		}

        private string GrindLocation()
        {
            return GetLocationForCrystal(GetCrystalToGather());
        }
		
		private int GrindLocationId()
        {
			return LocationDictionary.FirstOrDefault(x => x.Value == GrindLocation()).Key;
        }

		private int GrindLocationZoneId()
        {
			return ZoneDictionary.FirstOrDefault(x => x.Value == GrindLocation()).Key;
        }

		private string GetCrystalForLocation(string location)
        {
			return CrystalLocationDictionary.FirstOrDefault(x => x.Value == location).Key;
        }
		
        private string GetLocationForCrystal(string crystal)
        {
			return CrystalLocationDictionary.FirstOrDefault(x => x.Key == crystal).Value;
        }
		
        private string GetCurrentZoneNameViaId(uint zoneid)
        {
			return ZoneDictionary.FirstOrDefault(x => x.Key == zoneid).Value;
        }

		private bool AreWeInGrindZone()
		{
			return WorldManager.ZoneId == GrindLocationZoneId();
		}
		
		private bool InIdyllshire()
		{
			return WorldManager.ZoneId == 478;
		}
		
		private bool InDravanianHinterlands()
		{
			return WorldManager.ZoneId == 399;
		}
		
		private void MoveIntoDravanianHinterlands()
		{
			Logging.Write(LogColor, "[Crystal Hunter] Moving to Dravanian Hinterlands.");
			
			if (settings.IdyllshireAetheryteUnlocked)
			{
				MoveToIdyllshireAetheryte();
				
				// Interact with it
				Thread.Sleep(3000);
				GameObjectManager.GetObjectByNPCId(75).Interact();
				Thread.Sleep(3000);
				SelectString.ClickSlot(0);
				Thread.Sleep(3000);
				SelectString.ClickSlot(2); // Collector's Quarter Exit
				Thread.Sleep(15000);
			}
			else
			{
				Vector3 Location = new Vector3("142.4722, 207, 114.1796");
				Navigator.NavigationProvider.MoveTo(Location);
				MovementManager.SetFacing(Location);
				Thread.Sleep(2000);
				MovementManager.MoveForwardStart();
				Thread.Sleep(5000);
			}
		}
		
		private void MoveToIdyllshireAetheryte()
		{
			MovementManager.SetFacing(new Vector3("71.94617, 211.2611, -18.90594"));
			Thread.Sleep(5000);
			MovementManager.MoveForwardStart();
			Thread.Sleep(1000);
			MovementManager.Jump();
			Thread.Sleep(500);
			MovementManager.MoveForwardStop();
			Thread.Sleep(1000);
		}
		
        #endregion
    }
 }
